//
//  SDKSampleBaseViewController+ApiParams.swift
//  KakaoSDKSample
//
//  Created by kakao on 2023/07/19.
//  Copyright © 2023 apiteam. All rights reserved.
//

import UIKit

enum ApiKey : String, Codable {
    //user
    case loginWithKakaoTalk
    case loginWithKakaoAccount
    case loginWithKakaoAccountScopes
    case certLoginWithKakaoTalk
    case certLoginWithKakaoAccount
    case me
    case serviceTerms
    case revokeServiceTerms
    case revokeScopes
    case scopes
    
    //talk
    case friends
    
    //talk:channel
    case channels
    case addChannel
    case channelChat
}

// MARK: - ApiParam extension
extension SDKSampleBaseViewController {
    var apiParamsInfo: ApiParamsInfo {
        get {
            SDKSampleBaseViewController.loadApiParamsInfo()
        }
    }
    var defaultApiParamsInfo: ApiParamsInfo {
        get {
            SDKSampleBaseViewController.makeDefaultApiParamsInfo()
        }
    }
    
    static func makeDefaultApiParamsInfo() -> ApiParamsInfo {
        let defaultApiParamsInfo = ApiParamsInfo()
        
        defaultApiParamsInfo.info[ApiKey.loginWithKakaoTalk] =
        ApiParams(name:ApiKey.loginWithKakaoTalk.rawValue, paramItems: [
            ParamItem(key: "launchMethod", value:.singleSelectableItem(["CustomScheme", "UniversalLink"])),
            ParamItem(key: "channelPublicIds", value:.editableText(""), placeHolderValue: "',' separated string"),
            ParamItem(key: "serviceTerms", value:.editableText(""), placeHolderValue: "',' separated string"),
            ParamItem(key: "nonce", value:.editableText("test_nonce"))
        ])
        
        defaultApiParamsInfo.info[ApiKey.loginWithKakaoAccount] =
        ApiParams(name:ApiKey.loginWithKakaoAccount.rawValue, paramItems: [
            ParamItem(key: "prompts", value:.multiSelectableItem(["login", "create", "cert", "select_account"])),
            ParamItem(key: "loginHint", value:.editableText(""), placeHolderValue: "filled email"),
            ParamItem(key: "nonce", value:.editableText("test_nonce"))
        ])
        
        defaultApiParamsInfo.info[ApiKey.loginWithKakaoAccountScopes] =
        ApiParams(name:ApiKey.loginWithKakaoAccountScopes.rawValue, paramItems: [
            ParamItem(key: "scopes", value:.editableText(""), placeHolderValue: "',' separated string"),
            ParamItem(key: "nonce", value:.editableText("test_nonce"))
        ])
        
        defaultApiParamsInfo.info[ApiKey.certLoginWithKakaoTalk] =
        ApiParams(name:ApiKey.certLoginWithKakaoTalk.rawValue, paramItems: [
            ParamItem(key: "launchMethod", value:.singleSelectableItem(["CustomScheme", "UniversalLink"])),
            ParamItem(key: "prompts", value:.multiSelectableItem(["create"])),
            ParamItem(key: "serviceTerms", value:.editableText(""), placeHolderValue: "',' separated string"),
            ParamItem(key: "state", value:.editableText("test_state")),
            ParamItem(key: "nonce", value:.editableText("test_nonce")),
            ParamItem(key: "settleId", value:.editableText(AppDelegate.settleId()), editable: false)
        ])
        
        defaultApiParamsInfo.info[ApiKey.certLoginWithKakaoAccount] =
        ApiParams(name:ApiKey.certLoginWithKakaoAccount.rawValue, paramItems: [
            ParamItem(key: "prompts", value:.multiSelectableItem(["login", "create"])),
            ParamItem(key: "loginHint", value:.editableText(""), placeHolderValue: "filled email"),
            ParamItem(key: "nonce", value:.editableText("test_nonce")),
            ParamItem(key: "state", value:.editableText("test_state")),
            ParamItem(key: "settleId", value:.editableText(AppDelegate.settleId()), editable: false)
        ])
        
        defaultApiParamsInfo.info[ApiKey.me] =
        ApiParams(name:ApiKey.me.rawValue, paramItems: [
            ParamItem(key: "propertyKeys", value:.editableText(""), placeHolderValue: "',' separated string"),
        ])
        
        defaultApiParamsInfo.info[ApiKey.serviceTerms] =
        ApiParams(name:ApiKey.serviceTerms.rawValue, paramItems: [
            ParamItem(key: "result", value:.editableText("app_service_terms")),
            ParamItem(key: "tags", value: .editableText(""), placeHolderValue: "',' separated string")
        ])
        
        defaultApiParamsInfo.info[ApiKey.revokeScopes] =
        ApiParams(name:ApiKey.revokeScopes.rawValue, paramItems: [
            ParamItem(key: "scopes", value:.editableText(""), placeHolderValue: "',' separated string")
        ])
        
        defaultApiParamsInfo.info[ApiKey.scopes] =
        ApiParams(name:ApiKey.scopes.rawValue, paramItems: [
            ParamItem(key: "scopes", value:.editableText(""), placeHolderValue: "',' separated string")
        ])
        
        defaultApiParamsInfo.info[ApiKey.friends] =
        ApiParams(name:ApiKey.friends.rawValue, paramItems: [
            ParamItem(key: "offset", value:.editableText("")),
            ParamItem(key: "limit", value:.editableText("")),
            ParamItem(key: "order", value:.singleSelectableItem(["asc", "desc"])),
            ParamItem(key: "friendOrder", value:.singleSelectableItem(["nickname", "age", "favorite"])),
            
        ])
        
        defaultApiParamsInfo.info[ApiKey.channels] =
        ApiParams(name:ApiKey.channels.rawValue, paramItems: [
            ParamItem(key: "publicIds", value:.editableText("_ZeUTxl"), placeHolderValue: "',' separated string")
        ])
        
        defaultApiParamsInfo.info[ApiKey.addChannel] =
        ApiParams(name:ApiKey.channels.rawValue, paramItems: [
            ParamItem(key: "channelPublicId", value:.editableText(""), placeHolderValue: "channel public Id")
        ])
        
        defaultApiParamsInfo.info[ApiKey.channelChat] =
        ApiParams(name:ApiKey.channels.rawValue, paramItems: [
            ParamItem(key: "channelPublicId", value:.editableText(""), placeHolderValue: "channel public Id")
        ])
        
        defaultApiParamsInfo.info[ApiKey.revokeServiceTerms] = ApiParams(name: ApiKey.revokeServiceTerms.rawValue, paramItems: [ParamItem(key: "serviceTerms", value: .editableText("revoke_test"))])
        
        return defaultApiParamsInfo
    }
    
    static func loadApiParamsInfo() -> ApiParamsInfo {
        let loadedApiParamsInfo : ApiParamsInfo? = ParamUserDefaults.loadCodable()
        if let _apiParamsInfo = loadedApiParamsInfo {
            return _apiParamsInfo
        }
        else {
            return ApiParamsInfo()
        }
    }
    
    func saveApiParamsInfo() {
        ParamUserDefaults.saveCodable(data: self.apiParamsInfo)
    }    
    
    //settleId는 유저가 셋팅할수 없는 고정값이다.
    func checkExceptionalParamItem(_ apiParams: ApiParams?, _ defaultParams: ApiParams?) -> ApiParams? {
        if let newApiParams = apiParams {
            newApiParams.paramItems = newApiParams.paramItems.map({ paramItem in
                if paramItem.key == "settleId" {
                    paramItem.value = .editableText(AppDelegate.settleId())
                }
                return paramItem
            })
            
            if let defaultParams = defaultParams {
                newApiParams.paramItems = mergeWithDefaultParams(newApiParams, defaultParams)
            }
            
            return newApiParams
        }
        else {
            return nil
        }
    }
    
    func mergeWithDefaultParams(_ apiParams: ApiParams, _ defaultParams: ApiParams) -> [ParamItem] {
        let newApiParamsKey = apiParams.paramItems.map { item in item.key }
        let defaultApiParamsKey = defaultParams.paramItems.map{ item in item.key }
        
        let notContainedParams = defaultParams.paramItems.filter({ item in !newApiParamsKey.contains(item.key) })
        apiParams.paramItems.append(contentsOf: notContainedParams)
        
        return apiParams.paramItems.filter({ item in defaultApiParamsKey.contains(item.key) })
    }
    
    func apiParams(_ key:ApiKey) -> ApiParams? {
        if let apiParams = self.apiParamsInfo.info[key] {
            return checkExceptionalParamItem(apiParams, defaultApiParamsInfo.info[key])
        }
        else {
            return defaultApiParamsInfo.info[key]
        }
    }
    
    func presentPopup(vc:UIViewController) {
        vc.modalPresentationStyle = .overFullScreen
        vc.modalTransitionStyle = .crossDissolve
        if let topVC = UIApplication.getMostTopViewController() {
            DispatchQueue.main.async {
                topVC.present(vc, animated: false) {
                    print("presented")
                }
            }
        }
    }
    
    func showParamDialog(apiKey:ApiKey, completionDelay:CGFloat? = nil, completion:(([ParamItem]) -> Void)?) {
        guard let apiParams = self.apiParams(apiKey) else {
            print("\(apiKey) is not exist in ApiParams Data")
            return
        }
        let vc = SDKSampleParamViewController(apiParams: apiParams,
                                              completionDelay: completionDelay,
                                              paramViewCompletionHandler: { [weak self] (resultApiParams) in
            self?.apiParamsInfo.info[apiKey] = resultApiParams
            self?.saveApiParamsInfo()
            let paramItems = resultApiParams.paramItems
            
            if let completion = completion {
                completion(paramItems)
            }
        })
        self.presentPopup(vc: vc)
    }
}

